<?php
/*
Plugin Name: Bluetrait Event Viewer
Plugin URI: http://wordpress.org/extend/plugins/bluetrait-event-viewer/
Description: BTEV monitors events that occur in your WordPress install.
Version: 1.8.1
Author: Michael Dale
Author URI: http://www.bluetrait.com/
*/

define('BTEV_LOCKDOWN', FALSE);

//stop people from accessing the file directly and causing errors.
if (!function_exists('add_action')) die('You cannot run this file directly.');
	//don't break stuff if user tries to use this plugin on anything less than WordPress 2.0.0
	if (!function_exists('get_role')) {
		return;
	}
	//required for WordPress 2.5+
	global $btev_version;
	
	$btev_tb_events = $wpdb->prefix . 'btev_events';
	$btev_version = '1.8.1';
	
	//no longer used
	$btev_tb_site = $wpdb->prefix . 'btev_site';

	//setup the database, check/upgrade database
	btev_is_installed();
	btev_site();
	btev_upgrade();

	//take over trigger_error
	if (btev_get_config('set_error_report')) {
		set_error_handler('btev_error_report');
	}
	//this is used to override wp_login so that we can track failed logins better
	if (btev_get_config('override_wp_login')) {
		//WordPress 2.5 has hooks to check for failed logins
		if (version_compare($wp_version, '2.5', '<')) {
			//WordPress 1.5.0 doesn't allow a pluggable wp_login function
			if (!function_exists('wp_login')) {
				function wp_login($username, $password, $already_md5 = false) {
					global $wpdb, $error;

					if ( '' == $username )
						return false;

					if ( '' == $password ) {
						$error = __('<strong>ERROR</strong>: The password field is empty.');
						return false;
					}

					$login = get_userdatabylogin($username);

					if (!$login) {
						$error = __('<strong>ERROR</strong>: Invalid username.');
						do_action('btev_invalid_username', $username);
						return false;
					} else {
						// If the password is already_md5, it has been double hashed.
						// Otherwise, it is plain text.
						if ( ($already_md5 && md5($login->user_pass) == $password) || ($login->user_login == $username && $login->user_pass == md5($password)) ) {
							return true;
						} else {
							$error = __('<strong>ERROR</strong>: Incorrect password.');
							do_action('btev_invalid_password', $username);
							return false;
						}
					}
				}
			}
		}
	}
	if (btev_get_config('override_wp_mail')) {
		if (!function_exists('wp_mail')) {
			/*
			wp_mail has changed in wp 2.2 no longer need to override
			Note this compare won't match beta versions etc
			*/
			if (version_compare($wp_version, '2.2', '<')) {
				function wp_mail($to, $subject, $message, $headers = '') {
					if( $headers == '' ) {
						$headers = "MIME-Version: 1.0\n" .
						"From: wordpress@" . preg_replace('#^www\.#', '', strtolower($_SERVER['SERVER_NAME'])) . "\n" . 
						"Content-Type: text/plain; charset=\"" . get_option('blog_charset') . "\"\n";
					}
					do_action('btev_mail_sent', $to);
					return @mail($to, $subject, $message, $headers);
				}
			}
		}
	}
	
		
	//menu actions
	add_action('admin_menu', 'btev_admin_menu');
	add_action('admin_menu', 'btev_admin_menu_settings');
	add_action('admin_menu', 'btev_admin_menu_details');
	//events
	add_action('password_reset', 'btev_trigger_password_reset');
	add_action('delete_user', 'btev_trigger_delete_user');
	add_action('activate_' . plugin_basename(__FILE__), 'btev_trigger_activate_btev');
	add_action('deactivate_' . plugin_basename(__FILE__), 'btev_trigger_deactivate_btev');
	add_action('wp_login', 'btev_trigger_wp_login');
	add_action('lostpassword_post', 'btev_trigger_lostpassword_post');
	add_action('profile_update', 'btev_trigger_profile_update');
	add_action('add_attachment', 'btev_trigger_add_attachment');
	add_action('wp_logout', 'btev_trigger_wp_logout');
	add_action('user_register', 'btev_trigger_user_register');
	add_action('switch_theme', 'btev_trigger_switch_theme');
	//cron stuff
	add_filter('cron_schedules', 'btev_cron_more_reccurences');
	add_action('btev_cron_daily_tasks_hook', 'btev_cron_daily_tasks');
	add_action('btev_cron_weekly_tasks_hook', 'btev_cron_weekly_tasks');

	if (btev_get_config('display_on_dashboard')) {
		add_action('activity_box_end', 'btev_activity_box_end');
	}
	if (version_compare($wp_version, '2.5', '<')) {
		if (btev_get_config('override_wp_login')) {
			add_action('btev_invalid_username', 'btev_trigger_invalid_username');
			add_action('btev_invalid_password', 'btev_trigger_invalid_password');
		}
	}
	else {
		add_action('wp_login_failed', 'btev_trigger_login_error');
	}
	if (version_compare($wp_version, '2.2', '<')) {
		if (btev_get_config('override_wp_mail')) {
			add_action('btev_mail_sent', 'btev_trigger_mail_sent');
		}
	}
	else {
		add_action('phpmailer_init', 'btev_trigger_phpmailer_init');
	}
	add_action('init', 'btev_add_admin_cap');
	add_action('init', 'btev_uninstall');
	add_action('init', 'btev_request_handle', 100);
	add_action('load-plugins.php', 'btev_monitor_plugins');

//link to event viewer
function btev_subpanel_link() {
	return 'index.php?page=btev.php';
}

//link to event viewer settings
function btev_subpanel_settings_link() {
	return 'options-general.php?page=btev.php_settings';
}

//link to event details settings
function btev_subpanel_details_link() {
	return 'options-general.php?page=btev.php_event_details';
}


//checks to see if btev is installed
function btev_is_installed() {
	global $wpdb, $btev_tb_events;

	if($wpdb->get_var("show tables like '$btev_tb_events'") != $btev_tb_events) {
		btev_install_system();
		btev_trigger_error('Unable to locate BTEV database table. It has been automatically recreated, events in the log may have been lost.', E_USER_ERROR);
		btev_trigger_error("You can safely ignore the previous message if this is the first time you've activated this plugin.", E_USER_NOTICE);
		return false;
	}
	else {
		return true;
	}

}

//this function checks that the user is really trying to uninstall and if they have permission to (if so uninstall)
function btev_uninstall() {
	
	if (isset($_POST['submit_uninstall'])) {
		if (current_user_can('btev') && !BTEV_LOCKDOWN) {
			if (function_exists('check_admin_referer')) {
				check_admin_referer('btev-uninstall');
			}
			btev_uninstall_system();
		}
		else {
			btev_trigger_error("Unauthorised Uninstall Attempt of BTEV.", E_USER_WARNING);
		}
	}
}

//install database tables
function btev_install_system() {
	global $wpdb, $btev_version, $btev_tb_events;

	if($wpdb->get_var("show tables like '$btev_tb_events'") != $btev_tb_events) {

		$sql = "CREATE TABLE " . $btev_tb_events . " (
			`type` varchar(100) NOT NULL default '',
			`date` datetime NOT NULL default '0000-00-00 00:00:00',
			`source` text NOT NULL,
			`file_line` varchar(100) NOT NULL,
			`user_id` mediumint(9) unsigned NOT NULL default '0',
			`ip_address` varchar(100) NOT NULL default '',
			`event_id` int(11) NOT NULL auto_increment,
			`event_no` int(11) NOT NULL default '0',
			`description` TEXT NOT NULL,
			`trace` TEXT,
			UNIQUE KEY  (event_id)
		) DEFAULT CHARSET=utf8;";

		if (file_exists(ABSPATH . 'wp-admin/includes/upgrade.php')) {
			//wordpress 2.3+
			require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
		}
		else {
			//wordpress 2.2.x and lower
			require_once(ABSPATH . 'wp-admin/upgrade-functions.php');
		}
		
		dbDelta($sql);
	}
	
	//create config will do nothing if option already exists
	btev_create_config();	
	btev_site();
	btev_schedule_tasks();
	btev_trigger_error('Bluetrait Event Viewer ' . $btev_version . ' Has Been Successfully Installed.', E_USER_NOTICE);
	add_option('btev_installed', '1');
}

//this function does the uninstalling
function btev_uninstall_system() {
	global $wpdb, $btev_tb_events, $btev_tb_site;

	/*
	Deactivate Plugin
	*/
	$current = get_option('active_plugins');
	array_splice($current, array_search(plugin_basename(__FILE__), $current), 1 ); // Array-fu!
	update_option('active_plugins', $current);

	/*
	Drop Events Table
	*/
	if($wpdb->get_var("show tables like '$btev_tb_events'") == $btev_tb_events) {
		$wpdb->query("DROP TABLE $btev_tb_events");
	}
	
	/*
	Drop Site Table (shouldn't exist anyway)
	*/
	if($wpdb->get_var("show tables like '$btev_tb_site'") == $btev_tb_site) {
		$wpdb->query("DROP TABLE $btev_tb_site");
	}
	
	/*
	Delete Options from WordPress Table
	*/
	delete_option('btev_config');
	delete_option('btev_installed');
	
	/*
	Unschedule Cron Tasks
	*/
	if (function_exists('wp_clear_scheduled_hook')) {
		wp_clear_scheduled_hook('btev_cron_daily_tasks_hook');
		wp_clear_scheduled_hook('btev_cron_weekly_tasks_hook');
	}
	
	/*
	Redirect To Plugin Page
	*/
	wp_redirect('plugins.php?deactivate=true');
	

}

//upgrade database if needed
function btev_upgrade() {
	global $wpdb, $btev_version;

	if (!btev_get_config('version')) {
		//old version (0.3 and lower)
		btev_create_config();
		btev_to_04();
		btev_trigger_error("Your BTEV settings have been set back to the default settings. This won't happen in future upgrades.", E_USER_WARNING);
		btev_site();
	}
	else {
		if (btev_get_config('version') != $btev_version) {
			switch (btev_get_config('version')) {
				case '0.4':
					btev_to_05();
					btev_to_06();
					btev_to_1();
					btev_to_11();
					btev_to_12();
					btev_to_13();
					btev_to_14();
					btev_to_15();
					btev_to_16();
					btev_to_17();
					btev_to_18();
					btev_to_181();
				break;
				
				case '0.5':
					btev_to_06();
					btev_to_1();
					btev_to_11();
					btev_to_12();
					btev_to_13();
					btev_to_14();
					btev_to_15();
					btev_to_16();
					btev_to_17();
					btev_to_18();
					btev_to_181();
				break;
				
				case '0.6':
					btev_to_1();
					btev_to_11();
					btev_to_12();
					btev_to_13();
					btev_to_14();
					btev_to_15();
					btev_to_16();
					btev_to_17();
					btev_to_18();
					btev_to_181();
				break;
				
				case '1.0':
					btev_to_11();
					btev_to_12();
					btev_to_13();
					btev_to_14();
					btev_to_15();
					btev_to_16();
					btev_to_17();
					btev_to_18();
					btev_to_181();
				break;
				
				case '1.1':
					btev_to_12();
					btev_to_13();
					btev_to_14();
					btev_to_15();
					btev_to_16();
					btev_to_17();
					btev_to_18();
					btev_to_181();
				break;
				
				case '1.2':
					btev_to_13();
					btev_to_14();
					btev_to_15();
					btev_to_16();
					btev_to_17();
					btev_to_18();
					btev_to_181();
				break;
				
				case '1.3':
					btev_to_14();
					btev_to_15();
					btev_to_16();
					btev_to_17();
					btev_to_18();
					btev_to_181();
				break;
				
				case '1.4':
					btev_to_15();
					btev_to_16();
					btev_to_17();
					btev_to_18();
					btev_to_181();
				break;
				
				case '1.5':
					btev_to_16();
					btev_to_17();
					btev_to_18();
					btev_to_181();
				break;
				
				case '1.6':
					btev_to_17();
					btev_to_18();
					btev_to_181();
				break;
				
				case '1.7':
					btev_to_18();
					btev_to_181();
				break;
				
				case '1.8':
					btev_to_181();
				break;
				
				default:
					//cannot upgrade
			}
			//reload site details
			btev_site();
		}
	}
} 

//version specific upgrade function
function btev_to_04() {
	global $wpdb, $btev_version, $btev_tb_site;
	
	//$btev_tb_site is no longer used in 0.4 and higher
	if($wpdb->get_var("show tables like '$btev_tb_site'") == $btev_tb_site) {
		$wpdb->query("DROP TABLE $btev_tb_site");
		btev_trigger_error('BTEV database upgraded to version ' . $btev_version, E_USER_NOTICE);
	}
}

//version specific upgrade function
function btev_to_05() {

	$version = '0.5';
	btev_set_config('override_wp_mail', 0);
	btev_set_config('version', $version);
	btev_save_config();
	btev_trigger_error('BTEV database upgraded to version '. $version, E_USER_NOTICE);
	 
}

//version specific upgrade function
function btev_to_06() {

	$version = '0.6';
	btev_set_config('version', $version);
	btev_save_config();
	btev_trigger_error('BTEV database upgraded to version '. $version, E_USER_NOTICE);
	 
}

//version specific upgrade function
function btev_to_1() {

	$version = '1.0';
	btev_set_config('version', $version);
	btev_save_config();
	btev_trigger_error('BTEV database upgraded to version '. $version, E_USER_NOTICE);
	 
}

//version specific upgrade function
function btev_to_11() {

	$version = '1.1';
	btev_set_config('version', $version);
	btev_save_config();
	btev_trigger_error('BTEV database upgraded to version '. $version, E_USER_NOTICE);
	 
}

//version specific upgrade function
function btev_to_12() {

	$version = '1.2';
	btev_set_config('version', $version);
	btev_save_config();
	btev_trigger_error('BTEV database upgraded to version '. $version, E_USER_NOTICE);
	 
}

//version specific upgrade function
function btev_to_13() {

	$version = '1.3';
	btev_set_config('version', $version);
	btev_save_config();
	btev_trigger_error('BTEV database upgraded to version '. $version, E_USER_NOTICE);
	 
}
//version specific upgrade function
function btev_to_14() {

	$version = '1.4';
	btev_set_config('version', $version);
	btev_set_config('source_cropping', '');
	btev_save_config();
	btev_trigger_error('BTEV database upgraded to version '. $version, E_USER_NOTICE);
	 
}

//version specific upgrade function
function btev_to_15() {
	global $wpdb, $btev_tb_events;
	
	$version = '1.5';
	$wpdb->query("ALTER TABLE $btev_tb_events ADD COLUMN `file_line` varchar(100) NOT NULL");
	$wpdb->query("ALTER TABLE $btev_tb_events ADD COLUMN `trace` TEXT");
	btev_set_config('auto_prune', '0');
	btev_set_config('event_count', '10000');
	btev_set_config('version', $version);
	btev_save_config();
	btev_trigger_error('BTEV database upgraded to version '. $version, E_USER_NOTICE);
	 
}
//version specific upgrade function
function btev_to_16() {

	$version = '1.6';
	btev_set_config('version', $version);
	btev_save_config();
	btev_trigger_error('BTEV database upgraded to version '. $version, E_USER_NOTICE);
	 
}

//version specific upgrade function
function btev_to_17() {

	$version = '1.7';
	btev_set_config('version', $version);
	btev_save_config();
	btev_trigger_error('BTEV database upgraded to version '. $version, E_USER_NOTICE);
	 
}

//version specific upgrade function
function btev_to_18() {

	$version = '1.8';
	btev_set_config('version', $version);
	
	btev_set_config('external_access', '0');
	btev_set_config('external_access_key', '');
	
	btev_save_config();
	btev_trigger_error('BTEV database upgraded to version '. $version, E_USER_NOTICE);
	 
}

//version specific upgrade function
function btev_to_181() {

	$version = '1.8.1';
	btev_set_config('version', $version);
	
	//make sure cron stuff is up to date
	btev_schedule_tasks();
	
	btev_save_config();
	btev_trigger_error('BTEV database upgraded to version '. $version, E_USER_NOTICE);
	 
}


//lets get the details about btev out of the database
function btev_site() {
	global $btev_site;
	
	$btev_site = get_option('btev_config');
}

//returns a config value from the site array
function btev_get_config($config_name) {
	global $btev_site;
	if (!empty($btev_site)) {
		if (array_key_exists($config_name, $btev_site)) {
			return $btev_site[$config_name];
		}
		else {
			return false;
		}
	}
	else {
		return false;
	}
}

//sets a config value into the array
function btev_set_config($config_name, $config_value, $update_now = FALSE) {
	global $btev_site;
	
	$btev_site[$config_name] = $config_value;
	
	if ($update_now) {
		btev_save_config();
	}
}

//saves the site array back to the database
function btev_save_config() {
	global $btev_site;
	
	update_option('btev_config', $btev_site);	
}

//populates the site table with info
function btev_create_config() {
	global $btev_version;
	
	$site = array();
	//version 0.2
	$site['debug'] = 0;
	$site['log_all'] = 0;
	$site['display_on_dashboard'] = 1;
	$site['installed'] = current_time('mysql');
	$site['next_update_check'] = '';
	$site['last_update_response'] = '';
	$site['version'] = $btev_version;
	//version 0.3
	$site['set_error_report'] = 0;
	$site['override_wp_login'] = 0;
	//version 0.4
	$site['update_check_interval'] = 86400;
	//version 0.5
	$site['override_wp_mail'] = 0;
	//version 1.4
	$site['source_cropping'] = '';
	//version 1.5
	$site['auto_prune'] = 0;
	$site['event_count'] = 10000;
	//version 1.8
	$site['external_access'] = '0';
	$site['external_access_key'] = '';

	add_option('btev_config', $site, 'Bluetrait Event Viewer Config');
}

//get the current id of user logged in
function btev_user_id() {
	if (function_exists('wp_get_current_user')) {
		$user_object = wp_get_current_user();
		return $user_object->ID;
	}
	else {
		return 0;
	}
}

//add a role so that we can check if the user can "do stuff" (TM)
function btev_add_admin_cap() {

	$role = get_role('administrator');
	$role->add_cap('btev');

}

//html for event viewer settings
function btev_subpanel_settings() {
?>

<?php if (isset($_POST['submit'])) { 
	if (!BTEV_LOCKDOWN) {
		btev_set_config('display_on_dashboard', $_POST['btev_display_on_dashboard'] ? 1 : 0);
		btev_set_config('set_error_report', $_POST['btev_set_error_report'] ? 1 : 0);
		btev_set_config('log_all', $_POST['btev_log_all'] ? 1 : 0);
		btev_set_config('debug', $_POST['btev_debug'] ? 1 : 0);
		btev_set_config('override_wp_login', $_POST['btev_override_wp_login'] ? 1 : 0);
		btev_set_config('override_wp_mail', $_POST['btev_override_wp_mail'] ? 1 : 0);
		$btev_source_cropping = stripslashes($_POST['btev_source_cropping']);
		btev_set_config('source_cropping', $btev_source_cropping);
		btev_set_config('auto_prune', $_POST['btev_auto_prune'] ? 1 : 0);
		btev_set_config('event_count', (int) $_POST['btev_event_count']);
		btev_set_config('external_access', $_POST['btev_external_access'] ? 1 : 0);
		btev_set_config('external_access_key', $_POST['btev_external_access_key']);
		btev_save_config();
		btev_trigger_error("Bluetrait Event Viewer Settings Updated.", E_USER_NOTICE);
		?>
		<div id="message" class="updated fade"><p><strong><?php _e('Settings Updated.') ?></strong></p></div>
	<?php
	}
	else {
		btev_trigger_error("Unauthorised Update Attempt of BTEV Settings.", E_USER_WARNING);
	}
} ?>
<div class="wrap">
	<h2>Bluetrait Event Viewer Settings</h2>
	<form action="<?php echo btev_subpanel_settings_link(); ?>" method="post">
		<table class="form-table">
		<p>The function overrides (wp_login and wp_mail) allow BTEV to track more events. You should only need to disable these options if another plugin needs to override the same functions or you're running an unsupported version of WordPress.</p>
		<tr valign="top">
			<th scope="row">Display recent events</th>
			<td>
				<select name="btev_display_on_dashboard">
					<option value="0">Off</option>
					<option value="1" <?php if (btev_get_config('display_on_dashboard') == 1) { echo 'selected="selected"'; } ?>>On</option>
				</select>
				<br />
				On dashboard (frontpage)
			</td>
		</tr>
		
		<tr valign="top">
			<th scope="row">Source Cropping</th>
			<td>
				<input name="btev_source_cropping" type="text" size="50" value="<?php echo wp_specialchars(btev_get_config('source_cropping')); ?>" />
				<br />
				The text above is removed from the start of the source in the event viewer i.e /home/www/wordpress
			</td>
		</tr>
		
		<tr valign="top">
			<th scope="row">Auto Prune Events</th>
			<td>
				<select name="btev_auto_prune">
					<option value="0">Off</option>
					<option value="1" <?php if (btev_get_config('auto_prune') == 1) { echo 'selected="selected"'; } ?>>On</option>
				</select>
				<br />
				Requires WordPress 2.1+
			</td>
		</tr>
		
		<tr valign="top">
			<th scope="row">Events Kept</th>
			<td>
				<input name="btev_event_count" type="text" size="10" value="<?php echo wp_specialchars(btev_get_config('event_count')); ?>" />
				<br />
				Events kept until auto prune deletes them (auto prune must be enabled)
			</td>
		</tr>
		
		<tr valign="top">
			<th scope="row">External Access</th>
			<td>
				<select name="btev_external_access">
					<option value="0">Off</option>
					<option value="1" <?php if (btev_get_config('external_access') == 1) { echo 'selected="selected"'; } ?>>On</option>
				</select>
				<br />
				Allows external services to be used (such as RSS events feed)
			</td>
		</tr>
		
		<tr valign="top">
			<th scope="row">External Access Key</th>
			<td>
				<input name="btev_external_access_key" type="text" size="10" value="<?php echo wp_specialchars(btev_get_config('external_access_key')); ?>" />
				<br />
				Allows you to password protect external services
			</td>
		</tr>
		
		<tr valign="top">
			<th scope="row">RSS URL</th>
			<td>
				<?php
					$btev_rss_url = get_option('siteurl') . '/?btev_recent_event_rss';
					$btev_external = btev_get_config('external_access_key');
					if (!empty($btev_external)) {
						$btev_rss_url .= '&amp;btev_access_key=' . btev_get_config('external_access_key');
					}
				?>
				<input name="btev_rss_url" type="text" size="100" value="<?php echo wp_specialchars($btev_rss_url); ?>" />
				<br />
				Editing Disabled
			</td>
		</tr>
		
		
		<tr valign="top">
			<th scope="row">Override wp_login</th>
			<td>
				<select name="btev_override_wp_login">
					<option value="0">Off</option>
					<option value="1" <?php if (btev_get_config('override_wp_login') == 1) { echo 'selected="selected"'; } ?>>On</option>
				</select>
				<br />
				Uses a modified wp_login function to track failed logins, only required for WordPress 2.3 and lower
			</td>
		</tr>
		
		<tr valign="top">
			<th scope="row">Override wp_mail</th>
			<td>
				<select name="btev_override_wp_mail">
					<option value="0">Off</option>
					<option value="1" <?php if (btev_get_config('override_wp_mail') == 1) { echo 'selected="selected"'; } ?>>On</option>
				</select>
				<br />
				Uses a modified wp_mail function to track emails sent, only required for WordPress 2.1 and lower
			</td>
		</tr>
		
		</table>
		
		<h3>Advanced Settings</h3>
		
		<table class="form-table">
		<p>These settings may affect the performance of your blog and are only recommended if you know what you're doing (i.e writting plugins).</p>
		
		<tr valign="top">
			<th scope="row">PHP Error Reporting</th>
			<td>
				<select name="btev_set_error_report">
					<option value="0">Off</option>
					<option value="1" <?php if (btev_get_config('set_error_report') == 1) { echo 'selected="selected"'; } ?>>On</option>
				</select>
			</td>
		</tr>
		
		<tr valign="top">
			<th scope="row">Full Database Logging</th>
			<td>
				<select name="btev_log_all">
					<option value="0">Off</option>
					<option value="1" <?php if (btev_get_config('log_all') == 1) { echo 'selected="selected"'; } ?>>On</option>
				</select>
				<br />
				Logs all errors to the database from PHP Error Reporting (not recommended)
			</td>
		</tr>
		
		<tr valign="top">
			<th scope="row">Debugging</th>
			<td>
				<select name="btev_debug">
					<option value="0">Off</option>
					<option value="1" <?php if (btev_get_config('debug') == 1) { echo 'selected="selected"'; } ?>>On</option>
				</select>
				<br />
				Displays all events in HTML (not recommended and can break WordPress)
			</td>
		</tr>
		
		</table>
		<p class="submit"><input type="submit" name="submit" value="Submit"/></p>
	</form>
	
	<p><a href="<?php echo btev_subpanel_link(); ?>">&laquo; Events</a></p>
	<?php btev_display_updates(); ?>
	<div id="btev_uninstall">
		<script type="text/javascript">
		<!--
		function btev_uninstall() {
			if (confirm("Are you sure you wish to uninstall Bluetrait Event Viewer?")){
				return true;
			}
			else{
				return false;
			}
		}
		//-->
		</script>
		<form action="<?php echo btev_subpanel_settings_link(); ?>" method="post" onsubmit="return btev_uninstall(this);">
		<?php
			if (function_exists('wp_nonce_field')) {
				wp_nonce_field('btev-uninstall');
			}
			?>
			<p class="submit"><input type="submit" name="submit_uninstall" value="Uninstall" /> (This removes all BTEV database entries, including events and settings)</p>
		</form>
	</div>
</div>
<?php
}

//html for event viewer
function btev_subpanel() {
	global $wpdb, $btev_tb_events;

	if (isset($_POST['submit'])) {
		if (function_exists('check_admin_referer')) {
			check_admin_referer('btev-clear-logs');
		}
		if (!BTEV_LOCKDOWN) {
			$result = $wpdb->query("DELETE FROM $btev_tb_events");
			btev_trigger_error('Event Log Cleared.', E_USER_NOTICE);
		}
		else {
			btev_trigger_error("Unauthorised Deletion Attempt of BTEV Logs.", E_USER_WARNING);
		}
	}

	if (!empty($_GET['bt_page'])) {
		$page = (int) $_GET['bt_page'];
		if ($page == 0) $page = 1;
	}
	else {
		$page = 1;
	}
	
	if (!isset($_GET['bt_filter'])) {
		$filter = '';
	}
	else {
		$filter = $_GET['bt_filter'];
	}
	
	switch($filter) {
	
		case 1:
				$query = ' WHERE `type` = \'NOTICE\'';
				$filter = 1;
		break;
		
		case 2:
				$query = ' WHERE `type` = \'WARNING\'';
				$filter = 2;
		break;
		
		case 3:
				$query = ' WHERE `type` = \'ERROR\'';
				$filter = 3;
		break;
		
		case 4:
				$query = ' WHERE `type` = \'DEBUG\'';
				$filter = 4;
		break;
	
		default:
				$query = '';
				$filter = '';
	}
		$offset = $page * 100 - 100;
		
		if ($offset < 0) {
			$offset = 0;
		}
		
		$offset = (int) $offset;
		
		$next_page = $page + 1;
		$previous_page = $page - 1;
		if ($previous_page < 1) {
			$previous_page = 1;
		}
		if ($next_page < 1) {
			$next_page = 1;
		}

		$event_query = 'SELECT * FROM ' . $btev_tb_events . $query . ' ORDER BY event_id DESC LIMIT ' . $offset . ', 100 ';
		$event_result = $wpdb->get_results($event_query, 'ARRAY_A');
		$event_page_num = $wpdb->query($event_query);

		if ($event_page_num != 100) {
			$next_page = 1;		
			$link_text = 'First Page';
		}
		else {
			$link_text = 'Next Page';
		}
		
		if (!empty($filter)) {
			$page_filter = '&amp;bt_filter=' . $filter;
		}
		else {
			$page_filter = '';
		}
	?>
	<div class="wrap" style="max-width: 10000px;">
		<h2>Last 100 Events</h2>
		<p><a href="<?php echo btev_subpanel_link(); ?>">Display All</a> : <a href="<?php echo btev_subpanel_link(); ?>&amp;bt_filter=1">Display Notices</a> : <a href="<?php echo btev_subpanel_link(); ?>&amp;bt_filter=2">Display Warnings</a> : <a href="<?php echo btev_subpanel_link(); ?>&amp;bt_filter=3">Display Errors</a> : <a href="<?php echo btev_subpanel_link(); ?>&amp;bt_filter=4">Display Debug</a></p>
		<p><a href="<?php echo btev_subpanel_link(); ?>&amp;bt_page=<?php echo $previous_page . $page_filter; ?>">&laquo; Previous Page</a> <a href="<?php echo btev_subpanel_link(); ?>&amp;bt_page=<?php echo $next_page . $page_filter; ?>"><?php echo $link_text; ?> &raquo;</a></p>
		<table class="widefat">
			<thead>
				<tr>
					<th>ID</th>
					<th>Date/Time</th>
					<th>Type</th>
					<th>Source</th>
					<th>User ID</th>
					<th>IP address</th>
					<th>Description</th>
				</tr>
			</thead>
			<tbody>
				<?php 
				if ($event_page_num != 0) {
					$i = 1;
					foreach ($event_result as $event_array) {
						if(substr($event_array['source'], 0, strlen(btev_get_config('source_cropping'))) == btev_get_config('source_cropping')) {	
							$event_array['source'] = (substr($event_array['source'], strlen(btev_get_config('source_cropping'))));
						}
					?>
					<tr<?php if ($i % 2 == 0) echo ' class="alternate"'; ?>>
						<td><a href="<?php echo btev_subpanel_details_link(); ?>&amp;event_id=<?php echo wp_specialchars($event_array['event_id']); ?>"><?php echo wp_specialchars($event_array['event_id']); ?></a></td>
						<td><?php echo wp_specialchars($event_array['date']); ?></td>
						<td><?php echo wp_specialchars($event_array['type']); ?></td>
						<td><?php echo wp_specialchars($event_array['source']); ?></td>
						<td><?php echo '<a href="user-edit.php?user_id=' . wp_specialchars($event_array['user_id']) . '">' . wp_specialchars($event_array['user_id']). '</a>'; ?></td>
						<td><?php echo '<a href="http://services.bluetrait.org/lookup/?type=rdns&amp;ip_address=' . wp_specialchars($event_array['ip_address']) . '">' . wp_specialchars($event_array['ip_address']) . '</a>'; ?></td>
						<td><?php echo wp_specialchars($event_array['description']); ?></td>
					</tr>
					<?php 
					$i++; 
					}
				}					
				?>
			</tbody>
		</table>
		<p><a href="<?php echo btev_subpanel_link(); ?>&amp;bt_page=<?php echo $previous_page . $page_filter; ?>">&laquo; Previous Page</a> <a href="<?php echo btev_subpanel_link(); ?>&amp;bt_page=<?php echo $next_page . $page_filter; ?>"><?php echo $link_text; ?> &raquo;</a></p>
		<form method="post" action="">
			<?php
			if (function_exists('wp_nonce_field')) {
				wp_nonce_field('btev-clear-logs');
			}
			?>
			<p class="submit"><input type="submit" name="submit" value="Clear Logs" /></p>
		</form>
		<p><a href="<?php echo btev_subpanel_settings_link(); ?>">Settings &raquo;</a></p>
		<br />
		<br />
	</div>
	<?php
}

//html for event viewer
function btev_subpanel_event_details() {
	global $wpdb, $btev_tb_events;
	
	if (isset($_GET['event_id']) && !empty($_GET['event_id'])) {
		$event_id = (int) $_GET['event_id'];
	}
	else {
		$event_id = 0;
	}
	$event_query = "SELECT * FROM $btev_tb_events WHERE event_id = " . $wpdb->escape($event_id) . " LIMIT 1";
	$events = $wpdb->get_results($event_query, 'ARRAY_A');
	
	?>
	<div class="wrap">
		<h2>Event Details</h2>
		<div class="tablecontain">
<pre>
<?php
if (!empty($events)) {
$event = print_r($events[0], TRUE);
echo wp_specialchars($event);
}
else {
	?>
Event ID not found.

Please use <a href="<?php echo btev_subpanel_link(); ?>">Events</a> to select Event ID.
	<?php
}
?>
</pre>
		</div>
		<p><a href="<?php echo btev_subpanel_link(); ?>">&laquo; Events</a></p>
		<p style="float:right;"><a href="<?php echo btev_subpanel_settings_link(); ?>">Settings &raquo;</a></p>
	</div>	
<?php
	
}

//add recent events to dashboard (only for an admin)
function btev_activity_box_end() {
	global $wpdb, $btev_tb_events;
	
	if (current_user_can('btev')) {
		$event_query = "SELECT `date`, `description`, `type` FROM $btev_tb_events ORDER BY event_id DESC LIMIT 10";		
		$event_result = $wpdb->get_results($event_query, 'ARRAY_A');

		?>
		
		<h3>Recent Events <a href="<?php echo btev_subpanel_link(); ?>">&raquo;</a></h3>
			<table>
					<thead>
						<tr>
							<th>Date/Time</th>
							<th>Type</th>
							<th>Description</th>
						</tr>
					</thead>
					<tbody>
						<?php 
							$i = 1;
							foreach ($event_result as $event_array) { 
							?>
							<tr<?php if ($i % 2 == 0) echo ' class="alternate"'; ?>>
								<td><?php echo wp_specialchars($event_array['date']); ?></td>
								<td><?php echo wp_specialchars($event_array['type']); ?></td>
								<td><?php echo wp_specialchars($event_array['description']); ?></td>
							</tr>
							<?php 
							$i++; 
							}			
						?>
					</tbody>
			</table>
		<?php
	}
}

//trigger an event with this function. You can use this function for your own plugins. See http://www.bluetrait.com/page/btev/ for more details
function btev_trigger_error($error_string, $error_number = E_USER_ERROR, $file = __FILE__, $line = __LINE__) {
	do_action('btev_trigger_error', array(
	'error_string' => $error_string, 
	'error_number' => $error_number, 
	'error_file' => $file, 
	'error_line' => $line)
	);
	btev_error_report($error_number, $error_string, $file, $line);
}

//writes event to database and can be used for set_error_handler
function btev_error_report($errno, $errstr, $errfile, $errline) {
	global $wpdb, $btev_tb_events;
	
	if (btev_get_config('debug')) {
		echo '<br />' . wp_specialchars($errno) . ': ' . wp_specialchars($errstr) .  ' in <b>' . wp_specialchars($errfile) . '</b> on line <b>' . wp_specialchars($errline) . '</b>';
	}
	
	switch ($errno) {	
	  case E_USER_ERROR:
					/*
					Get the backtrace here
					*/
					if (function_exists('debug_print_backtrace')) {
						ob_start();
						debug_print_backtrace();
						$trace = ob_get_contents();
						ob_end_clean();
					}
					else {
						$trace = 'Backtrace is not supported in PHP4, please upgrade to PHP5.';
					}
					
					$result = $wpdb->query(
					"INSERT into $btev_tb_events (`type`, `event_no`, `date`, `source`, `user_id`, `ip_address`, `description`, `file_line`, `trace`) values 
					('ERROR',
					'" . $wpdb->escape($errno) . "',
					'" . $wpdb->escape(current_time('mysql')) ."',
					'" . $wpdb->escape($errfile) . "',
					'" . $wpdb->escape(btev_user_id()) . "',
					'" . $wpdb->escape(btev_ip_address()) . "',
					'" . $wpdb->escape($errstr) . "',
					'" . $wpdb->escape($errline) . "',
					'" . $wpdb->escape($trace) . "')
					");		
	   break;
	  case E_USER_WARNING:
					$result = $wpdb->query(
					"INSERT into $btev_tb_events(`type`, `event_no`, `date`, `source`, `user_id`, `ip_address`, `description`, `file_line`) values 
					('WARNING',
					'" . $wpdb->escape($errno) . "',
					'" . $wpdb->escape(current_time('mysql')) ."',
					'" . $wpdb->escape($errfile) . "',
					'" . $wpdb->escape(btev_user_id()) . "',
					'" . $wpdb->escape(btev_ip_address()) . "',
					'" . $wpdb->escape($errstr) . "',
					'" . $wpdb->escape($errline) . "')
					");		
	   break;
	  case E_USER_NOTICE:
					$result = $wpdb->query(
					"INSERT into $btev_tb_events (`type`, `event_no`, `date`, `source`, `user_id`, `ip_address`, `description`, `file_line`) values 
					('NOTICE',
					'" . $wpdb->escape($errno) . "',
					'" . $wpdb->escape(current_time('mysql')) ."',
					'" . $wpdb->escape($errfile) . "',
					'" . $wpdb->escape(btev_user_id()) . "',
					'" . $wpdb->escape(btev_ip_address()) . "',
					'" . $wpdb->escape($errstr) . "',
					'" . $wpdb->escape($errline) . "')
					");		
	   break;
	  default:
	  		if(btev_get_config('log_all')) {
					/*
					Get the backtrace here
					*/
					if (function_exists('debug_print_backtrace')) {
						ob_start();
						debug_print_backtrace();
						$trace = ob_get_contents();
						ob_end_clean();
					}
					else {
						$trace = 'Backtrace is not supported in PHP4, please upgrade to PHP5.';
					}
					
					$result = $wpdb->query(
					"INSERT into $btev_tb_events (`type`, `event_no`, `date`, `source`, `user_id`, `ip_address`, `description`, `file_line`, `trace`) values 
					('DEBUG',
					'" . $wpdb->escape($errno) . "',
					'" . $wpdb->escape(current_time('mysql')) ."',
					'" . $wpdb->escape($errfile) . "',
					'" . $wpdb->escape(btev_user_id()) . "',
					'" . $wpdb->escape(btev_ip_address()) . "',
					'" . $wpdb->escape($errstr) . "',
					'" . $wpdb->escape($errline) . "',
					'" . $wpdb->escape($trace) . "')
					");			
			}
	   break;
	}
}

//checks if there is a newer version of btev
function btev_version_check() {
	global $wpdb, $btev_tb_event;

	if (current_time('mysql') > btev_get_config('next_update_check')) {
		$r = new btev_HTTPRequest('http://update.bluetrait.org/btev/version.html');
		$response = addslashes($r->DownloadToString());
		$next_update = btev_now('Y-m-d H:i:s', btev_get_config('update_check_interval'));
		btev_set_config('next_update_check', $next_update); 
		btev_save_config();
		
		if (!empty($response)) {	
			btev_set_config('last_update_response', $response); 
			btev_save_config();
		}
		else {
			$response = btev_get_config('last_update_response');
		}
	}
	else {
		$response = btev_get_config('last_update_response');	
	}
	return $version = explode(',', $response);
}

//displays if there is a newer version on the settings page
function btev_display_updates() {
	
	$version = btev_version_check();
	
	if (version_compare(btev_get_config('version'), $version[0]) === 1 || version_compare(btev_get_config('version'), $version[0]) === 0) {
		?><strong>You're currently running the latest version, <?php echo btev_get_config('version'); ?>.</strong><?php
	}
	else {
		?><div id="message" class="updated fade"><p><strong>There is a new version of this software available (You're running <?php echo btev_get_config('version'); ?> the latest is <?php echo wp_specialchars($version[0]); ?>). 
		You can download the latest version from <a href="<?php echo wp_specialchars($version[2]); ?>">here</a>. 
		You can view the release notes <a href="<?php echo wp_specialchars($version[1]); ?>">here</a>.
		</strong></p></div>
		<?php if (!empty($version[3])) { ?>
			<br /><br />
			<strong><?php echo wp_specialchars($version[3]);?></strong>
		<?php }
	}
}

//basic date function. Should be able to use a wordpress one though.
function btev_now($format = 'Y-m-d H:i:s', $add_seconds = 0) {

	$base_time = time() + $add_seconds + 3600 * get_option('gmt_offset');
	
	switch($format) {
	
		case 'Y-m-d H:i:s':
			return gmdate('Y-m-d H:i:s', $base_time);
		break;
		
		case 'H:i:s':
			return gmdate('H:i:s', $base_time);
		break;
		
		case 'Y-m-d':
			return gmdate('Y-m-d', $base_time);
		break;
		
		case 'Y':
			return gmdate('Y', $base_time);
		break;
		
		case 'm':
			return gmdate('m', $base_time);
		break;
		
		case 'd':
			return gmdate('d', $base_time);
		break;
	
	}
}

//adds the event viewer to the dashboard submenu
function btev_admin_menu() {
	if (function_exists('add_submenu_page')) {
		add_submenu_page('index.php', 'Event Viewer', 'Event Viewer', 8, basename(__FILE__), 'btev_subpanel');
	}
}

//adds the event viewer settings to the options submenu
function btev_admin_menu_settings() {
	if (function_exists('add_options_page')) {
		add_options_page('Event Viewer Settings', 'Event Viewer Settings', 8, basename(__FILE__) . '_settings', 'btev_subpanel_settings');
	}
}
//adds the event viewer settings to the options submenu
function btev_admin_menu_details() {
	if (function_exists('add_options_page')) {
		add_options_page('Event Viewer Details', 'Event Viewer Details', 8, basename(__FILE__) . '_event_details', 'btev_subpanel_event_details');
	}
}

//returns the users ip address
function btev_ip_address() {
	return $_SERVER['REMOTE_ADDR'];
}

//used for downloading the string containing version info
class btev_HTTPRequest
{
   var $_fp;        // HTTP socket
   var $_url;        // full URL
   var $_host;        // HTTP host
   var $_protocol;    // protocol (HTTP/HTTPS)
   var $_uri;        // request URI
   var $_port;        // port
  
   // scan url
   function _scan_url()
   {
       $req = $this->_url;
      
       $pos = strpos($req, '://');
       $this->_protocol = strtolower(substr($req, 0, $pos));
      
       $req = substr($req, $pos+3);
       $pos = strpos($req, '/');
       if($pos === false)
           $pos = strlen($req);
       $host = substr($req, 0, $pos);
      
       if(strpos($host, ':') !== false)
       {
           list($this->_host, $this->_port) = explode(':', $host);
       }
       else
       {
           $this->_host = $host;
           $this->_port = ($this->_protocol == 'https') ? 443 : 80;
       }
      
       $this->_uri = substr($req, $pos);
       if($this->_uri == '')
           $this->_uri = '/';
   }
  
   // constructor
   function btev_HTTPRequest($url)
   {
	   $this->_url = $url;
       $this->_scan_url();
   }
  
   // download URL to string
   function DownloadToString()
   {
       $crlf = "\r\n";
	   $response = '';
      
       // generate request
       $req = 'GET ' . $this->_uri . ' HTTP/1.0' . $crlf
           .    'Host: ' . $this->_host . $crlf . 'User-Agent: btev/' . btev_get_config('version') . $crlf
           .    $crlf;
      
       // fetch
       
	   if ($this->_fp = @fsockopen(($this->_protocol == 'https' ? 'ssl://' : '') . $this->_host, $this->_port, $errno, $errstr, 3)) {
	   		fwrite($this->_fp, $req);
			while(is_resource($this->_fp) && $this->_fp && !feof($this->_fp))
           	$response .= fread($this->_fp, 1024);
			fclose($this->_fp);

			// split header and body
		   $pos = strpos($response, $crlf . $crlf);
		   if($pos === false)
			   return($response);
		   $header = substr($response, 0, $pos);
		   $body = substr($response, $pos + 2 * strlen($crlf));
		  
		   // parse headers
		   $headers = array();
		   $lines = explode($crlf, $header);
		   foreach($lines as $line)
			   if(($pos = strpos($line, ':')) !== false)
				   $headers[strtolower(trim(substr($line, 0, $pos)))] = trim(substr($line, $pos+1));
		  
		   // redirection?
		   if(isset($headers['location']))
		   {
			   $http = new btev_HTTPRequest($headers['location']);
			   return($http->DownloadToString($http));
		   }
		   else
		   {
			   return($body);
		   }
       }
	   else {
	   		btev_trigger_error('Unable to contact BTEV update server.', E_USER_WARNING);
			 return;
	   }
   }
}

/*
	The following part handles the cron section for Bluetrait Event Viewer
===========================================================================================================================================
*/
//this is where we schedule the cron tasks
function btev_schedule_tasks() {

	if (function_exists('wp_next_scheduled')) {
		if (!wp_next_scheduled('btev_cron_daily_tasks_hook')) {
			wp_schedule_event(0, 'daily', 'btev_cron_daily_tasks_hook');
		}
		if (!wp_next_scheduled('btev_cron_weekly_tasks_hook')) {
			wp_schedule_event(0, 'weekly', 'btev_cron_weekly_tasks_hook');
		}
		return true;
	}
	else {
		return false;
	}

}

//add the weekly option to the wordpress cron
function btev_cron_more_reccurences() {
	return array(
		'weekly' => array('interval' => 604800, 'display' => 'Weekly')
	);
}

//this is where we can now put any functions we want to run every day
function btev_cron_daily_tasks() {
	if(btev_get_config('auto_prune')) {
		btev_auto_prune();
	}
}

//this is where we can now put any functions we want to run every week
function btev_cron_weekly_tasks() {
	
	$version = btev_version_check();
	
	if (version_compare(btev_get_config('version'), $version[0]) === 1 || version_compare(btev_get_config('version'), $version[0]) === 0) {

	}
	else {
		btev_trigger_error('There is a new version of Bluetrait Event Viewer, version ' .
		wp_specialchars($version[0]) . ' you are running ' . wp_specialchars(btev_get_config('version')), E_USER_WARNING);
	}
}

//this function removes old entries from the event log if enabled
function btev_auto_prune() {
	global $wpdb, $btev_tb_events;
	
	$event_count_query = "SELECT count(*) as `count` FROM $btev_tb_events";
	$event_result = $wpdb->get_results($event_count_query, 'ARRAY_A');
	
	$delete_count = btev_get_config('event_count');
	$event_count = $event_result[0]['count'];
	
	if ($event_count > $delete_count) {
		$events_to_delete = $event_count - $delete_count;
		$events_to_delete = (int) $events_to_delete;
		
		$event_delete_query = "DELETE FROM $btev_tb_events ORDER BY date LIMIT $events_to_delete";
		$count = $wpdb->query($event_delete_query);
	}
	else {
		$count = 0;
	}

	btev_trigger_error('BTEV Auto Prune has finished and deleted "' . (int) $count . '" events.', E_USER_NOTICE);
}

/*
===========================================================================================================================================
*/

/*
This function makes it possible to monitor activation/deactivation of other plugins
*/
function btev_monitor_plugins() {

	$current = get_plugins();

	foreach($current as $plugin_file => $plugin_data) {
		if ($plugin_file == plugin_basename(__FILE__)) continue;
			add_action('deactivate_' . $plugin_file, 'btev_trigger_deactivate_plugin');
			add_action('activate_' . $plugin_file, 'btev_trigger_activate_plugin');
	}	
}

/*
These functions trigger events to be written to the database uses Wordpress hooks.
An add_action trigger also needs to be setup for each event to work. See btev_load();
*/

function btev_trigger_password_reset() {
	btev_trigger_error('Password Reset', E_USER_NOTICE);
	return;
}
function btev_trigger_delete_user($user_id) {
	btev_trigger_error('User Deleted ID: ' . $user_id, E_USER_NOTICE);
	return;
}
function btev_trigger_activate_btev() {
	btev_trigger_error('Bluetrait Event Viewer activated.', E_USER_NOTICE);
	return;
}
function btev_trigger_deactivate_btev() {
	if (!BTEV_LOCKDOWN) {
		btev_trigger_error('Bluetrait Event Viewer deactivated.', E_USER_NOTICE);
	}
	else {
		add_action('shutdown', 'btev_lockdown_reactivate');
	}
	return;
}
function btev_lockdown_reactivate() {
	//nope BTEV isn't going away
	btev_trigger_error("Unauthorised Deactivation Attempt of BTEV.", E_USER_WARNING);
	$current = get_option('active_plugins');
	if (!isset($current[plugin_basename(__FILE__)])) {
		$current[] = plugin_basename(__FILE__);
		sort($current);
		update_option('active_plugins', $current);
	}
	return;
}
function btev_trigger_wp_login($user_login) {
	btev_trigger_error('Login Successful: "' . $user_login . '"', E_USER_NOTICE);
	return;
}
function btev_trigger_wp_logout($user_login) {
	btev_trigger_error('Logout Successful.', E_USER_NOTICE);
	return;
}
function btev_trigger_user_register($user_id) {
	btev_trigger_error('User Added ID: ' . $user_id, E_USER_NOTICE);
	return;
}
function btev_trigger_lostpassword_post() {
	btev_trigger_error('A password reset was requested.', E_USER_NOTICE);
	return;
}
function btev_trigger_profile_update($user_id) {
	btev_trigger_error('A user has updated their profile.', E_USER_NOTICE);
	return;
}
function btev_trigger_add_attachment($id) {
	$file = get_attached_file($id);
	btev_trigger_error('File Uploaded: "' . $file . '"', E_USER_NOTICE);
	return;
}
function btev_trigger_login_errors($errors) {
	btev_trigger_error($errors, E_USER_WARNING);
	return $errors;
}
function btev_trigger_invalid_username($username) {
	btev_trigger_error('Login Failed: Unknown User "' . $username . '"', E_USER_WARNING);
	return;
}
function btev_trigger_invalid_password($username) {
	btev_trigger_error('Login Failed: Incorrect Password For "' . $username . '"', E_USER_WARNING);
	return;
}
function btev_trigger_switch_theme($theme) {
	btev_trigger_error('Theme Switched To: "' . $theme . '"', E_USER_NOTICE);
	return;
}
//for Wordpress 2.1 or lower
function btev_trigger_mail_sent($to) {
	btev_trigger_error('An Email Was Send To: "' . $to . '"', E_USER_NOTICE);
	return;
}
//for Wordpress 2.2 or higher
function btev_trigger_phpmailer_init(&$phpmailer) {
	$array = $phpmailer->to;
	foreach($array as $index => $value) {
		if (!empty($value[0])) {
			btev_trigger_error('An Email Was Send To: "' . $value[0] . '"', E_USER_NOTICE);
		}
	}
	return $phpmailer;
}
/*
These two plugins are a little hacky but seem to work fine. 
*/
function btev_trigger_activate_plugin() {
	$plugin = trim($_GET['plugin']);
	btev_trigger_error('Plugin activated: "'.$plugin.'"', E_USER_NOTICE);
	return;
}
function btev_trigger_deactivate_plugin() {
	if (isset($_GET['plugin'])) {
		$plugin = trim($_GET['plugin']);
		btev_trigger_error('Plugin deactivated: "'.$plugin.'"', E_USER_NOTICE);
	}
	elseif ($_GET['action'] == 'deactivate-all') {
		//are we Deactivating All?
		btev_trigger_error('All plugins have been deactivated.', E_USER_NOTICE);
	}
	
	return;
}
//for WordPress 2.5 or higher
function btev_trigger_login_error($username) {
	
	$user = get_userdatabylogin($username);
	
	if ( !$user || ($user->user_login != $username) ) {
		btev_trigger_error('Login Failed: Unknown User "' . $username . '"', E_USER_WARNING);
	}
	else {
		btev_trigger_error('Login Failed: Incorrect Password For "' . $username . '"', E_USER_WARNING);
	}
}

//outputs last 10 events via RSS
function btev_recent_events_rss($access_key = '') {
	global $wpdb, $btev_tb_events;
	
	//should probably move this into a function or something
	$btev_rss_url = get_option('siteurl') . '/?btev_recent_event_rss';

	header('Content-type: text/xml; charset=' . get_option('blog_charset'), true);
	echo '<?xml version="1.0" encoding="' . get_option('blog_charset') . '"?' .'>';
	?>
	<rss version="2.0" 
	xmlns:content="http://purl.org/rss/1.0/modules/content/"
	xmlns:wfw="http://wellformedweb.org/CommentAPI/"
	xmlns:dc="http://purl.org/dc/elements/1.1/"
	xmlns:atom="http://www.w3.org/2005/Atom"
	>
		<channel>
			<title>BTEV: <?php bloginfo_rss('name'); ?></title>
			<link><?php bloginfo_rss('url') ?></link>
			<description>Recent Events for <?php bloginfo_rss('name'); ?></description>
			<generator>Bluetrait Event Viewer</generator>
			<atom:link href="<?php echo wp_specialchars($btev_rss_url); ?>" rel="self" type="application/rss+xml" />
			<?php
			if (btev_get_config('external_access') && $access_key == btev_get_config('external_access_key')) {
				$event_query = "SELECT `event_id`, `date`, `description`, `type` FROM $btev_tb_events ORDER BY event_id DESC LIMIT 10";		
				$event_result = $wpdb->get_results($event_query, 'ARRAY_A');
				foreach ($event_result as $event_array) { 
					?>
					<item>
						<title><?php echo wp_specialchars($event_array['type']); ?>: <?php echo wp_specialchars($event_array['description']); ?></title>
						<link><?php echo get_option('siteurl'); ?>/wp-admin/<?php echo btev_subpanel_details_link(); ?>&amp;event_id=<?php echo (int) $event_array['event_id']; ?></link>
						<pubDate><?php echo mysql2date('D, d M Y H:i:s +0000', date('Y-m-d H:i:s', strtotime($event_array['date'])), false); ?></pubDate>
						<description><![CDATA[Date/Time: <?php echo wp_specialchars($event_array['date']); ?>]]></description>
					</item>
					<?php
				}
			}
			else {
			?>
			<item>
				<title>RSS Feed Disabled or Incorrect Password</title>
			</item>
			<?php
			}
			?>
		</channel>
	</rss>
	<?php
	die();
}

//hooks into Wordpress so that we can output "custom content"
function btev_request_handle() {

	if (isset($_GET['btev_recent_event_rss'])) {
		if (isset($_GET['btev_access_key'])) {
			btev_recent_events_rss($_GET['btev_access_key']);
		}
		else {
			btev_recent_events_rss();
		}
	}

}
?>